#include <IVst.h>
#include <IControllerBase.h>

#ifdef Q_OS_WIN
#include <qt_windows.h>
#include <process.h>
#include <shellapi.h>
#define SLEEP_TIME 8
#endif

static int argc=2;
static char **argv;
int plugin_instances=0;

#ifdef Q_OS_WIN
unsigned int __stdcall threadInit(void *args)
{
    IVst *plugin = (IVst*)args;

    plugin->setWaiting(true);

    #if 1
    while(plugin->running())
    {
        plugin->app()->processEvents();
        Sleep(SLEEP_TIME);
    }
    #else
    plugin->app()->exec();
    #endif

    plugin->setWaiting(false);

    return 0;
}
#endif

QGuiApplication* IVst::_app = 0;

QGuiApplication *IVst::app()
{
    if (_app)
        return _app;

    #ifndef QT_NO_DEBUG
    qDebug() << "Qt version: " QT_VERSION_STR;
    qDebug("QApplication pointer: %p", _app);
    #endif

    #ifdef Q_OS_WIN
    LPWSTR *argvv = CommandLineToArgvW(GetCommandLineW(), &argc);
    argv = new char*[argc];

    for (int i=0;i<argc;i++)
    {
        argv[i] = qstrdup(QString::fromStdWString(argvv[i]).toLocal8Bit().constData());
        #ifdef QT_DEBUG
        qDebug("argc: %d argv[%d]: %s", argc, i, argv[i]);
        #endif
    }
    #endif

    _app = new QGuiApplication(argc, argv);

    return _app;
}

IVst::IVst(audioMasterCallback audioMaster, int programs, int parameters) :
    QObject(0),
    AudioEffectX(audioMaster, programs, parameters),
    _controller(0),
    _running(true),
    _waiting(false),
    _updatingDisplay(true)
{
    programsAreChunks(true);

    plugin_instances++;

    #ifdef Q_OS_WIN
    unsigned int id;
    Qt::HANDLE h = (Qt::HANDLE) _beginthreadex(NULL, 0, threadInit, this, 0, &id);
    #endif
}

IVst::~IVst()
{
    if (editor)
    {
        delete editor;
        editor = 0;
    }

    if (_controller)
    {
        delete _controller;
        _controller=0;
    }

    _running=false;

    plugin_instances--;

    if (plugin_instances==0)
    {
        _app->exit();

        #ifdef Q_OS_WIN
        while(_waiting)
        {
            Sleep(SLEEP_TIME);
        }
        #endif

        delete _app;

        _app = 0;
    }
}

IVstEditor *IVst::gui()
{
    return (IVstEditor*) editor;
}

void IVst::setController(IControllerBase* controller)
{
    if (controller)
    {
        _controller = controller;

        connect(_controller, SIGNAL(updateHostDisplay()), SLOT(updateHostDisplay()));
    }
}

void IVst::setEditorWithDimensions(int width, int height)
{
    if (!editor)
    {
        if (_controller)
        {
            editor = new IVstEditor(this, _controller);
        }
        else
        {
            qWarning("Cannot instantiate editor without controller");
        }
    }
}

VstInt32 IVst::getChunk(void **data, bool isPreset)
{
    Q_UNUSED(isPreset)

    return (VstInt32) _chunk.size();
}

VstInt32 IVst::setChunk(void *data, VstInt32 size, bool isPreset)
{
    Q_UNUSED(isPreset)

    _updatingDisplay = true;

    return size;
}

VstInt32 IVst::getProgram()
{
    return 0;
}

void IVst::setProgram(VstInt32 program)
{
    if (!_updatingDisplay)
    {
    }
}

void IVst::getProgramName(char* text)
{
    QString name = "N/A";

    vst_strncpy(text, name.toUtf8().data(), name.size());
}

void IVst::setProgramName(char* name)
{
}

float IVst::getParameter(VstInt32 index)
{
    return 0;
}

void IVst::setParameter(VstInt32 index, float value)
{
}

void IVst::getParameterLabel(VstInt32 index, char* label)
{
    QString name = "";
    vst_strncpy(label, name.toUtf8().data(), name.size());
}

void IVst::getParameterDisplay(VstInt32 index, char* text)
{
    QString name = "";

    vst_strncpy(text, name.toUtf8().data(), name.size());
}

void IVst::getParameterName(VstInt32 index, char* text)
{
    QString name;

    vst_strncpy(text, name.toUtf8().data(), name.size());
}

bool IVst::getProgramNameIndexed(VstInt32 category, VstInt32 index, char *text)
{
    Q_UNUSED(category)

    QString name;

    vst_strncpy(text, name.toUtf8().data(), name.size());

    return true;
}

void IVst::setSampleRate(float sampleRate)
{
    AudioEffectX::setSampleRate(sampleRate);
}

void IVst::setBlockSize(VstInt32 blockSize)
{
    AudioEffectX::setBlockSize(blockSize);
}

bool IVst::running()
{
    return _running;
}

bool IVst::waiting()
{
    return _waiting;
}

void IVst::setWaiting(bool value)
{
    _waiting = value;
}

void IVst::updateHostDisplay()
{
    if (!_updatingDisplay)
    {
        _updatingDisplay = true;

        updateDisplay();
    }
}

void IVst::parameterChanged(int id, QVariant value, bool ui)
{
}

void IVst::sendNoteOnToHost(int note, int velocity)
{
}

void IVst::sendNoteOffToHost(int note)
{
}
