#ifndef IVST_H
#define IVST_H

#include <array>

#include <public.sdk/source/vst2.x/audioeffectx.h>

#include <QObject>
#include <QByteArray>
#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QQmlContext>
#include <QQuickView>
#include <QDebug>
#include <QtQml>
#include <QtPlugin>
#include <QQmlExtensionPlugin>
#include <QTimer>

#include <IVstEditor.h>

#ifdef USE_CONTROLLER_BASE
class IControllerBase;
#else
class IController;
#endif
class IVst : public QObject, public AudioEffectX
{
    Q_OBJECT

public:
    static QGuiApplication* _app;

    IVst(audioMasterCallback audioMaster, int numPrograms, int numParameters);
    ~IVst();

    static QGuiApplication* app();

    IVstEditor* gui();

    #ifdef USE_CONTROLLER_BASE
    virtual void setController(IControllerBase *controller);
    #else
    virtual void setController(IController *controller);
    #endif

    void setEditorWithDimensions(int width, int height);

    virtual VstInt32 getChunk(void **data, bool isPreset);
    virtual VstInt32 setChunk(void *data, VstInt32 byteSize, bool isPreset);

    virtual VstInt32 getProgram();
    virtual void setProgram(VstInt32 program);

    virtual void getProgramName(char* name);
    virtual void setProgramName(char* name);

    virtual float getParameter(VstInt32 index);
    virtual void setParameter(VstInt32 index, float value);

    virtual void getParameterLabel(VstInt32 index, char* label);
    virtual void getParameterDisplay(VstInt32 index, char* text);
    virtual void getParameterName(VstInt32 index, char* text);
    virtual bool getProgramNameIndexed(VstInt32 category, VstInt32 index, char* text);
	
    virtual void setSampleRate(float sampleRate);
    virtual void setBlockSize(VstInt32 blockSize);
	
    bool running();

    bool waiting();
    void setWaiting(bool value);

private slots:
    void parameterChanged(int id, QVariant value, bool changedByUi=true);
    void updateHostDisplay();

    void sendNoteOnToHost(int note, int velocity);
    void sendNoteOffToHost(int note);

protected:
    Qt::HANDLE _thread;

    #ifdef USE_CONTROLLER_BASE
    IControllerBase* _controller;
    #else
    VstIntPtr vendorSpecific(VstInt32 opcode, VstIntPtr lArg2, void* ptrArg, float) final;
    VstInt32 canDo(const char* feature);
    IController* _controller;
    #endif
    QTimer _timer;
    VstEvents* _events;

    int _updateDisplayProgram;

    bool _running;
    bool _waiting;
    bool _updatingDisplay;
    bool _supportNksFormat;

    QByteArray _chunk;

private:
    #ifndef USE_CONTROLLER_BASE
    bool getVendorSpecificPropertyInfo(unsigned nNKSOpcode, size_t& nDataSize, bool& bWritable);
    bool getVendorSpecificProperty(unsigned nNKSOpcode, void* pData);
    bool setVendorSpecificProperty(unsigned nOpcode, const void* pData, size_t nDataSize);
    bool notifyNKSHost(VstInt32 nNKSOpcode, void* pData, size_t nDataSize);
    #endif
};

#endif
