#include "IControllerBase.h"
#include "IVst.h"
#include "IVstEditor.h"

IVstEditor::IVstEditor(AudioEffect* effect, IControllerBase *controller) :
    QObject((IVst*)effect),
    _controller(controller),
    _quickView(nullptr),
    #ifdef Q_OS_WIN
    _window(nullptr),
    #endif
    _hasFocus(false)
{
    effect->setEditor(this);
}

IVstEditor::~IVstEditor()
{
    #ifdef Q_OS_WIN
    if (_window)
    {
        delete _window;

        _window = nullptr;
    }
    #endif
}

bool IVstEditor::open(void *ptr)
{
    _ptr = ptr;

    AEffEditor::open(_ptr);

    #ifdef Q_OS_WIN
    if (!_window)
    {
        _window = new QWindow();
        _window->setFlags(Qt::Tool | Qt::FramelessWindowHint);
        _window->setGeometry(0, 0, 640, 480);
    }
    #endif

    if (!_quickView)
    {
        _quickView = new QQuickView();

        _quickView->rootContext()->setContextProperty("controller", _controller);
        _quickView->setSource(QUrl("qrc:/qml/plugin.qml"));
        _quickView->setGeometry(0, 0, 640, 480);

        if (_controller->handleFocus())
            _quickView->setFlags(Qt::Tool | Qt::FramelessWindowHint | Qt::WindowDoesNotAcceptFocus);
        else
            _quickView->setFlags(Qt::Tool | Qt::FramelessWindowHint);

        _controller->setQuickView(_quickView);
    }

    #ifdef Q_OS_WIN
    SetParent((HWND)_quickView->winId(), (HWND)ptr);
    _window->setProperty("_q_embedded_native_parent_handle", QVariant((WId)ptr));
    _quickView->setParent(_window);
    _quickView->show();
    _window->show();
    #endif

    QMetaObject::invokeMethod(_controller, "editorReady", Qt::DirectConnection);

    return true;
}

void IVstEditor::close()
{
    if (_quickView)
    {
        _quickView->hide();

        #ifdef Q_OS_WIN
        _quickView->setParent(0);
        SetParent((HWND)_window->winId(), (HWND)0);
        _window->setProperty("_q_embedded_native_parent_handle", QVariant((WId)0));

        if (_window)
        {
            delete _window;

            _window=0;
        }
        #endif
    }

    AEffEditor::close();
}

bool IVstEditor::getRect(ERect **rect)
{
    static ERect r = {
        (VstInt16)0,
        (VstInt16)0,
        (VstInt16)480,
        (VstInt16)640
    };

    *rect = &r;

    return true;
}

QQuickItem* IVstEditor::rootObject()
{
    return _quickView->rootObject();
}

QQmlContext* IVstEditor::rootContext()
{
    return _quickView->rootContext();
}

bool IVstEditor::onKeyUp(VstKeyCode &keyCode)
{
    Q_UNUSED(keyCode)

    return false;
}

bool IVstEditor::onKeyDown(VstKeyCode &keyCode)
{
    return false;
}
